<?php
/**
 * @file
 * Cassette Drush commands.
 */

/**
 * Implementation of hook_drush_command().
 */
function cassette_drush_command() {
  $items = array();

  $items['cassette-kit'] = array(
    'description' => 'Set up a starter kit.',
    'arguments' => array(
      'name' => 'A name for your theme.',
    ),
    'options' => array(
      'machine' => '[a-z, 0-9] A machine-readable name for your theme.',
    ),
    'examples' => array(
      'drush cassette-create "My theme name"',
    ),
  );
  $items['cassette-columns'] = array(
    'description' => 'Set up a columnar grid system.',
    'arguments' => array(
      'theme_name' => 'Machine-readable name of the targeted theme.',
      'column_amount' => '[0-9] The amount of columns. Default: 9',
      'gutter_width' => '[0-9] The space between columns, in pixels. Default: 20',
      'wrapper_width' => '[0-9] Width of a wrapper (in pixels) which will be used to calculate widths of all possible columns. Default: 950',
    ),
    'examples' => array(
      'drush cassette-columns mythemename',
      'drush cassette-columns mythemename 8',
      'drush cassette-columns mythemename 8 10 940',
    ),
  );

  return $items;
}

/**
 * Command: Set up a starter kit.
 */
function drush_cassette_kit($name = 'My theme') {
  $machine_name = drush_get_option('machine');
  $machine_name = ($machine_name) ? $machine_name : preg_replace('/[^a-z0-9]+/', '', strtolower($name));

  $cassette_path = drush_locate_root() . '/' . drupal_get_path('theme', 'cassette');

  // From Cassette's location, we move up one dir and construct the path
  // where our new theme will be created.
  $_cassette_path = split('/', $cassette_path);
  array_pop($_cassette_path);
  $theme_path = join('/', $_cassette_path) . '/' . $machine_name;

  // Make a fresh copy of the original starter kit. Ignore layout-columns.css,
  // as it is optional and set up via another command separately.
  drush_op('cassette_copy_recursive', "$cassette_path/STARTERKIT", $theme_path, '/^(\.(\.)?|CVS|SVN|\.DS_Store|layout-columns\.css)$/');

  // Rename the info file and fill in the theme name.
  drush_op('rename', "$theme_path/STARTERKIT.info.txt", "$theme_path/$machine_name.info");
  drush_op('cassette_file_str_replace', "$theme_path/$machine_name.info", '= STARTERKIT', "= $name");

  // Replace all occurences of 'STARTERKIT' with the machine name of our sub theme.
  drush_op('cassette_file_str_replace', "$theme_path/theme-settings.php", 'STARTERKIT', $machine_name);
  drush_op('cassette_file_str_replace', "$theme_path/template.php", 'STARTERKIT', $machine_name);

  // Notify user of the newly created theme.
  drush_print(dt('Created starter kit "!name" in !path.',
    array(
      '!name' => $name,
      '!path' => $theme_path,
    )
  ));

  // Clear the cache so Drupal will detect our theme changes.
  drush_op('drupal_flush_all_caches');
}

/**
 * Command: Set up a columnar grid system.
 */
function drush_cassette_columns($theme_name, $column_amount = 9, $gutter_width = 20, $wrapper_width = 950) {
  // Check if the theme exists.
  $theme_path = drupal_get_path('theme', $theme_name);
  if (!$theme_path) {
    drush_die(dt('Could not find theme "!name".', array('!name' => $theme_name)));
  }

  $root = drush_locate_root();
  $theme_path = "$root/$theme_path";
  $cassette_path = $root . '/' . drupal_get_path('theme', 'cassette');

  // Format span selectors.
  $span_selectors_array = array();
  for ($i = 1; $i < $column_amount; $i++) {
    $span_selectors_array[$i] = 'WRAPPER_SELECTOR .span-' . $i;
  }
  $span_selectors = join(",\n", $span_selectors_array);

  // Calculate widths for every possible column span.
  $column_styles_array = array();
  foreach ($span_selectors_array as $span_amount => $span_selector) {
    $span_width =
      ((($wrapper_width - ($gutter_width * ($column_amount - 1))) / $column_amount) * $span_amount) +
      ($gutter_width * ($span_amount - 1));
    $span_width = floor($span_width);

    $column_styles_array[$span_amount] = <<<END
$span_selector {
  width: {$span_width}px;
}
END;
  }
  $column_styles = join("\n", $column_styles_array);

  // Copy the column template to our theme and replace tokens with our data.
  drush_op('copy', "$cassette_path/STARTERKIT/css/layout-columns.css", "$theme_path/css/layout-columns-{$column_amount}.css");
  drush_op('cassette_file_str_replace', "$theme_path/css/layout-columns-{$column_amount}.css",
    array(
      'COLUMN_AMOUNT',
      'SPAN_SELECTORS',
      'COLUMN_STYLES',
      'WRAPPER_SELECTOR',
      'WRAPPER_WIDTH',
      'GUTTER_WIDTH',
    ),
    array(
      $column_amount,
      $span_selectors,
      $column_styles,
      '.wrapper-' . $column_amount,
      $wrapper_width . 'px',
      $gutter_width . 'px',
    )
  );

  // Add a reference to the new CSS file in our info file. This is quick and
  // dirty for now.
  drush_op('cassette_file_str_replace', "$theme_path/$theme_name.info",
    'stylesheets[all][] = css/layout.css',
    "stylesheets[all][] = css/layout.css\nstylesheets[all][] = css/layout-columns-$column_amount.css"
  );

  // Notify user of the newly created CSS file.
  drush_print(dt('Created a !column_amount column grid system with !gutter_width gutters and a total width of !wrapper_width in !path',
    array(
      '!column_amount' => $column_amount,
      '!gutter_width' => $gutter_width . 'px',
      '!wrapper_width' => $wrapper_width . 'px',
      '!path' => "$theme_path/css/layout-columns-$column_amount.css",
    )
  ));

  // Clear the cache so Drupal will detect our theme changes.
  drush_op('drupal_flush_all_caches');
}

/**
 * Copy a directory recursively.
 */
function cassette_copy_recursive($source_dir, $target_dir, $ignore = '/^(\.(\.)?|CVS|SVN|\.DS_Store)$/') {
  if (!is_dir($source_dir)) {
    drush_die(dt('Could not find dir !dir.', array('!dir' => $source_dir)));
  }
  $dir = opendir($source_dir);
  @mkdir($target_dir);
  while($file = readdir($dir)) {
    if (!preg_match($ignore, $file)) {
      if (is_dir("$source_dir/$file")) {
        cassette_copy_recursive("$source_dir/$file", "$target_dir/$file", $ignore);
      }
      else {
        copy("$source_dir/$file", "$target_dir/$file");
      }
    }
  }
  closedir($dir);
}

/**
 * Replace strings in a file.
 */
function cassette_file_str_replace($file_path, $find, $replace) {
  $file_contents = file_get_contents($file_path);
  $file_contents = str_replace($find, $replace, $file_contents);
  file_put_contents($file_path, $file_contents);
}
